@@ -3,89 +3,21 @@
 
 # User helpers --------------------------------------------------------------------------------
 
-
-#' Find a keyword in the whole database
-#' 
-#' Find a keyword in all names and labels of a list of datasets. 
-#'
-#' @param keyword the keyword to search for. Can handle regular expressions (see examples).
-#' @param data the lookup dataframe where to search the keyword. Can be set using `edc_options(edc_lookup=my_data)`, which is done automatically when calling [read_trialmaster()].
-#' @param ignore_case should case differences be ignored in the match? Default to `TRUE`.
-#'
-#' @return a tibble
-#' @export
-#' @examples 
-#' \dontrun{
-#' path = system.file("extdata/Example_Export_SAS_XPORT_2022_08_25_15_16.zip", 
-#'                    package="EDCimport", mustWork=TRUE)
-#' w = read_trialmaster(path, verbose=FALSE)
-#' 
-#' find_keyword("patient")
-#' 
-#' #with regex
-#' find_keyword("patient$")
-#' find_keyword("\\d")
-#' find_keyword("(Trial|Form) Name")
-#' find_keyword("\\(") #you need to escape special characters
-#' }
-#' @importFrom cli cli_warn
-#' @importFrom dplyr any_of filter mutate pull select where
-#' @importFrom purrr map2_chr map2_dbl
-#' @importFrom stringr str_detect str_trim
-#' @importFrom tidyr unite unnest
-find_keyword = function(keyword, data=edc_lookup(), ignore_case=TRUE){
-  invalid=names2=labels2=x=NULL
-  f = if(isTRUE(ignore_case)) tolower else identity
-  f2 = function(x,y) map2_chr(x, y, ~if(.y) {.x} else {f(.x)})
-  keyword = f(str_trim(keyword))
-  tmp = data %>% 
-    select(-any_of(c("nrow", "ncol", "n_id", "rows_per_id"))) %>% 
-    unnest(c(names, labels)) %>% 
-    mutate(
-      labels=unlist(labels), 
-      invalid=is_invalid_utf8(labels),
-      names2=f2(names, invalid),
-      labels2=f2(labels, invalid),
-    ) %>% 
-    filter(str_detect(names2, keyword) | str_detect(labels2, keyword)) %>% 
-    select(-names2, -labels2) %>% 
-    mutate(prop_na = map2_dbl(dataset, names, ~{
-      if(!exists(.x, envir=globalenv())) return(NA)
-      x=get(.x, envir=globalenv())[[.y]]
-      mean(is.na(x))
-    })) %>% 
-    select(-where(~all(is.na(.x))))
-  if(nrow(tmp)==0) return(NULL)
-  
-  if(isTRUE(ignore_case) && any(tmp[["invalid"]])){
-    cols = tmp %>% filter(invalid) %>% unite("x", c("dataset", "names"), sep="$") %>% pull(x)
-    cli_warn(c("Some columns have labels containing non UTF8 characters. {.arg ignore_case} has been ignored for these.", 
-               i="Columns: {.code {cols}}.",
-               i='For instance, try to run: {.code attr({cols[1]}, "label")}.'), 
-             class="find_keyword_utf8_warning")
-  }
-  
-  tmp %>% 
-    select(-invalid)
-}
-
-
-
 #' Format factor levels as Yes/No
 #'
 #' Format factor levels as arbitrary values of Yes/No (with Yes always first) while **leaving untouched** all vectors that contain other information. 
 #'
 #' @param x a vector of any type/class.
-#' @param input list of values to be considered as "yes" and "no".
+#' @param input list of values to be considered as "yes", "no", and `NA`.
 #' @param output the output factor levels.
-#' @param strict whether to match the input strictly or use [stringr::str_detect] to find them.
+#' @param strict whether to match the input strictly or use [stringr::str_detect] to find them. Can also be "ignore_case" to just ignore the case.
 #' @param mutate_character whether to turn characters into factor.
 #' @param fail whether to fail if some levels cannot be recoded to yes/no.
 #'
 #' @return a factor, or `x` untouched.
 #' @export
 #' @importFrom cli cli_abort
-#' @importFrom dplyr case_when
+#' @importFrom dplyr case_match case_when 
 #' @importFrom stringr str_detect
 #'
 #' @examples 
@@ -99,7 +31,7 @@ find_keyword = function(keyword, data=edc_lookup(), ignore_case=TRUE){
 #'   b=sample(c("Oui", "Non"), size=N, replace=TRUE),
 #'   c=sample(0:1, size=N, replace=TRUE),
 #'   d=sample(c(TRUE, FALSE), size=N, replace=TRUE),
-#'   e=sample(c("1-Yes", "0-No"), size=N, replace=TRUE),
+#'   e=sample(c("1-Yes", "0-No", "2-NA"), size=N, replace=TRUE),
 #'   
 #'   y=sample(c("aaa", "bbb", "ccc"), size=N, replace=TRUE),
 #'   z=1:N,
@@ -107,20 +39,23 @@ find_keyword = function(keyword, data=edc_lookup(), ignore_case=TRUE){
 #'  
 #' x          
 #' #y and z are left untouched (or throw an error if fail=TRUE)   
-#' sapply(x, fct_yesno, fail=FALSE)
+#' sapply(x, fct_yesno, fail=FALSE, simplify=FALSE)
 #' 
 #' # as "1-Yes" is not in `input`, x$e is untouched/fails if strict=TRUE
 #' fct_yesno(x$e)
 #' fct_yesno(x$e, strict=TRUE, fail=FALSE) 
 #' fct_yesno(x$e, output=c("Ja", "Nein"))
 fct_yesno = function(x, 
-                     input=list(yes=c("Yes", "Oui"), no=c("No", "Non")), 
+                     input=list(yes=c("Yes", "Oui"), no=c("No", "Non"), na=c("NA", "")),
                      output=c("Yes", "No"),
                      strict=FALSE,
                      mutate_character=TRUE, 
                      fail=TRUE){
   assert_class(input, "list")
-  assert(setequal(names(input), c("yes", "no")))
+  default_input = list(yes=c("Yes", "Oui"), no=c("No", "Non"), na=c("NA", ""))
+  missing_names = setdiff(names(default_input), names(input))
+  input[missing_names] = default_input[missing_names]
+  assert(setequal(names(input), c("yes", "no", "na")))
   
   if (!inherits(x, c("logical", "numeric", "integer", "character", "factor"))) return(x)
   if (is.character(x) && isFALSE(mutate_character)) return(x)
@@ -136,13 +71,18 @@ fct_yesno = function(x,
   }
   
   if (!isFALSE(strict)) {
-    fun = if(strict=="ignore_case")tolower else identity
+    fun = if(strict=="ignore_case") tolower else identity
     is_yes = fun(x) %in% fun(input$yes)
     is_no  = fun(x) %in% fun(input$no)
+    is_na  = fun(x) %in% fun(input$na)
   } else {
+    input = map(input, ~case_match(.x, ""~"^$", .default=.x))
     is_yes = str_detect(tolower(x), paste(tolower(input$yes), collapse="|"))
     is_no  = str_detect(tolower(x), paste(tolower(input$no ), collapse="|"))
+    is_na  = str_detect(tolower(x), paste(tolower(input$na ), collapse="|"))
   }
+  x[is_na] = NA
+  
   if (any(is_yes&is_no, na.rm=TRUE)) {
     v = x[!is.na(x) & is_yes & is_no]
     cli_abort("Values that are both yes and no: {.val {v}}", 
@@ -177,13 +117,15 @@ fct_yesno = function(x,
 #'
 #' @return `df` with less columns
 #' @export
+#' @importFrom dplyr across all_of distinct everything n_distinct select summarise where
 #'
 #' @examples
-#' tm = edc_example_ae()
-#' tm$ae %>% names
-#' tm$ae %>% select_distinct() %>% names
-#' tm$ae %>% select_distinct(.by=subjid) %>% names
-#' @importFrom dplyr across all_of distinct everything n_distinct select summarise where
+#' db = edc_example()
+#' db$ae %>% colnames()
+#' #`crfname` has one level for the whole dataset
+#' db$ae %>% select_distinct() %>% colnames()
+#' #`n_ae` has one level per patient
+#' db$ae %>% select_distinct(.by=subjid) %>% colnames()
 select_distinct = function(df, .by) {
   a = df %>% 
     summarise(across(everything(), function(.x) n_distinct(.x, na.rm=TRUE)), 
@@ -197,91 +139,6 @@ select_distinct = function(df, .by) {
 }
 
 
-#' Get a table with the latest date for each patient
-#' 
-#' This function search for date columns in every tables and returns the latest date 
-#' for each patient with the variable it comes from. Useful in survival analysis to get 
-#' the right censoring time.
-#'
-#' @param except the datasets/columns that should not be searched. Example: a scheduled visit for which the patient may have died before attending should not be considered.
-#' @param with_ties in case of tie, whether to return the first `origin` (FALSE) or all the origins that share this tie (TRUE).
-#' @param numeric_id set to FALSE if the patient ID column is not numeric
-#' @param prefer preferred origins in the event of a tie. Usually the followup table.
-#' @param warn_if_future whether to show a warning about dates that are after the extraction date
-#'
-#' @return a dataframe
-#' @export
-#'
-#' @examples
-#' tm = edc_example_plot()
-#' load_list(tm)
-#' lastnews_table()
-#' lastnews_table(except="db3")
-#' lastnews_table(except="db3$date9")
-#' lastnews_table(prefer="db2") 
-#' @importFrom cli cli_abort
-#' @importFrom dplyr arrange filter mutate rowwise select slice_max ungroup where
-#' @importFrom purrr discard discard_at imap list_rbind
-#' @importFrom tidyr pivot_longer
-lastnews_table = function(except=NULL, with_ties=FALSE, numeric_id=TRUE, 
-                          prefer=NULL,
-                          warn_if_future=TRUE) {
-  subjid_cols = get_subjid_cols()
-  rtn = get_datasets(envir=parent.frame()) %>% 
-    discard_at(as.character(except)) %>% 
-    imap(~{
-      if(!is.data.frame(.x) || !any(subjid_cols %in% names(.x))) return(NULL)
-      a = .x %>% 
-        select(subjid = any_of2(subjid_cols), where(is.Date)) %>% 
-        mutate(subjid = as.character(subjid))
-      if(ncol(a)<=1) return(NULL) #only subjid
-      a %>% 
-        pivot_longer(-subjid) %>% 
-        filter(!is.na(value)) %>% 
-        mutate(origin_label=unlist(get_label(.x)[name]),
-               dataset=.y)
-    }) %>% 
-    discard(is.null) %>% 
-    list_rbind()  %>% 
-    select(subjid, last_date=value, origin_data=dataset, origin_col=name, origin_label) %>% 
-    mutate(origin = paste0(origin_data, "$", origin_col))
-  if(nrow(rtn)==0){
-    cli_abort("No data with dates could be found, verify your export settings.")
-  }
-  if(numeric_id && can_be_numeric(rtn$subjid)) {
-    rtn$subjid = as.numeric(as.character(rtn$subjid))
-  }
-  
-  rtn = rtn %>% 
-    filter(!origin %in% except) %>% 
-    filter(!origin_data %in% except) %>% 
-    filter(!origin_col %in% except) %>% 
-    slice_max(last_date, by=subjid, with_ties=TRUE) %>% 
-    arrange(order(mixedorder(subjid)))
-  
-  if(!with_ties){
-    rtn = rtn %>% 
-      rowwise() %>%  
-      mutate(prefered = which(prefer %in% c(origin, origin_data, origin_col)) %0% Inf) %>% 
-      ungroup() %>% 
-      arrange((prefered)) %>% 
-      select(-prefered, -origin) %>% 
-      slice_max(last_date, by=subjid, with_ties=FALSE) %>% 
-      arrange(order(mixedorder(subjid)))
-  }
-  
-  if(exists("datetime_extraction")){
-    if(any(rtn$last_date > as.Date(datetime_extraction)) && warn_if_future) {
-      rtn %>% 
-        filter(last_date>as.Date(datetime_extraction)) %>% 
-        edc_data_warn("Date of last news after the extraction date", issue_n=NA)
-    }
-  }
-  
-  rtn
-}
-
-
 #' Shows how many code you wrote
 #'
 #' @param main the main R file, which sources the other ones
@@ -290,16 +147,19 @@ lastnews_table = function(except=NULL, with_ties=FALSE, numeric_id=TRUE,
 #' @return Nothing
 #' @export
 #' @importFrom cli cli_inform
+#' @importFrom fs file_exists path
 #' @importFrom purrr map_dbl
 #' @importFrom rlang set_names
 #' @importFrom stringr str_extract str_subset
 edc_inform_code = function(main="main.R", Rdir="R/"){
+  if(!file_exists(main)) main = path(Rdir, main)
   assert_file_exists(main)
   sources = readLines(main) %>% 
-    str_subset(Rdir) %>% str_subset("^ *#", negate=TRUE) %>% str_extract('"(.*)"', group=TRUE) %>% 
+    str_subset(Rdir) %>% str_subset("^ *#", negate=TRUE) %>% 
+    str_extract('"(.*)"', group=TRUE) %>% 
     set_names()
   
-  rslt = sources %>% map_dbl(~readLines(.x) %>% length())
+  rslt = sources %>% map_dbl(~readLines(.x, warn=FALSE) %>% length())
   
   cli_inform("Sourcing {length(rslt)} files in {.path {main}} for a total 
              of {sum(rslt)} lines of code.")
@@ -332,54 +192,125 @@ save_sessioninfo = function(path="check/session_info.txt", with_date=TRUE){
   invisible(TRUE)
 }
 
-#' Harmonize the subject ID of the database
+
+#' Set the project name
 #' 
-#' Turns the subject ID columns of all datasets into a factor containing levels for all 
-#' the subjects of the database. Avoid problems when joining tables, and some checks can
-#' be performed on the levels.
+#' Set or override the project name
 #'
-#' @param datalist a list of dataframes
-#' @param preprocess an optional function to modify the subject ID column, for example `as.numeric()`. See examples.
-#' @param col_subjid the names of the columns holding the subject ID (as character)
+#' @param db the [edc_database]
+#' @param name the project name
 #'
-#' @return datalist, with subject id modified
+#' @returns nothing
+#' @export
+#'
+#' @examples
+#' db = edc_example() %>% 
+#'  set_project_name("My great project")
+#' edc_lookup()
+set_project_name = function(db, name){
+  lookup = db$.lookup %>% 
+    structure(project_name = name)
+  .update_lookup(new=lookup)
+  db
+}
+
+
+# Joins ---------------------------------------------------------------------------------------
+
+
+#' @noRd
+#' @keywords internal
+#' @importFrom cli cli_abort
+#' @importFrom dplyr all_of everything select setdiff
+#' @importFrom purrr keep
+#' @importFrom rlang as_label caller_call
+.edc_join = function(type){
+  dplyr_join = switch(type, 
+                left=dplyr::left_join, right=dplyr::right_join, 
+                full=dplyr::full_join, inner=dplyr::inner_join)
+  
+  function(x, y, by=NULL, suffix=NULL, cols=everything(), remove_dups=FALSE){
+    subjid_col = get_subjid_cols() %>% intersect(names(x)) %>% intersect(names(y))
+    if(is.null(by)) by = subjid_col
+    if(is.null(suffix)){
+      suffix = c("", paste0("_", as_label(caller_call(0)[[3]])))
+    }
+    
+    if(length(subjid_col)==0 && is.null(by)){
+      cli_abort(c("Could not find a common primary key for {.arg x} and {.arg y}",
+                  i="Primary key in current database: {.val {get_subjid_cols()}}"),
+                class="edc_subjid_not_found")
+    }
+    y = y %>% select(any_of(by), !!enquo(cols))
+    if(isTRUE(remove_dups)){
+      common_col = intersect(names(x), names(y)) %>% 
+        setdiff(by) %>% 
+        keep(~setequal(x[[.x]], y[[.x]]))
+      y = y %>% select(-all_of(common_col))
+    }
+
+    .safe_join(x, y, by=by, suffix=suffix, fun=dplyr_join) %>% 
+      copy_label_from(y) %>% 
+      copy_label_from(x)
+  }
+}
+
+#' @noRd
+#' @keywords internal
+#' @importFrom dplyr across all_of mutate
+.safe_join = function(x, y, by, suffix, fun){
+  j = tryCatch(fun(x, y, by=by, suffix=suffix),
+               error=function(e) e)
+  if(inherits(j, "dplyr_error_join_incompatible_type")){
+    f = function(d){
+      d %>% mutate(across(all_of(by), ~copy_label_from(as.character(.x), .x)))
+    }
+    x = f(x)
+    y = f(y)
+    j = fun(x, y, by=by, suffix=suffix)
+  } else if(inherits(j, "error")){
+    stop(j)
+  }
+  j
+}
+
+
+#' Join within the EDCimport framework
+#' 
+#' Perform a join with default `by` to the Subject ID and default suffix to the 
+#' name of the `y` dataset. See `[dplyr::mutate-joins]` for the description of the
+#' join logic.
+#'
+#' @param x,y Data frames to join
+#' @param by The key to join on, as character. Defaults to `get_subjid_cols()`
+#' @param suffix The disambiguation suffix. Defaults to the actual name of the `y` dataset.
+#' @param cols <[tidy-select][dplyr::dplyr_tidy_select]> The columns to select in `y` before joining.
+#' @param remove_dups Whether to remove columns in `y` that already exist in `x`.
+#'
+#' @returns a dataframe
 #' @export
-#' @importFrom dplyr across any_of mutate select
-#' @importFrom purrr discard_at keep map modify
 #'
 #' @examples
 #' db = edc_example()
-#' db$db0 = head(db$db0, 10)
-#' db$db0$SUBJID %>% head()
-#' db = harmonize_subjid(db)
-#' db$db0$SUBJID %>% head()
-#' db = harmonize_subjid(db, preprocess=function(x) paste0("#", x))
-#' db$db0$SUBJID %>% head()
-harmonize_subjid = function(datalist, preprocess=NULL, 
-                            col_subjid=get_subjid_cols()){
-  if(is.null(preprocess)) preprocess = identity
-  assert_class(preprocess, "function")
-  
-  all_subjid = datalist %>% 
-    keep(is.data.frame) %>% 
-    discard_at(".lookup") %>% 
-    map(~select(.x, any_of(col_subjid))) %>% 
-    unlist() %>% 
-    as.character() %>% 
-    preprocess() %>% 
-    unique() %>% 
-    mixedsort()
-  
-  a= datalist %>% 
-    modify(function(df){
-      if(!is.data.frame(df)) return(df)
-      df %>% 
-        mutate(across(any_of(get_subjid_cols()), 
-                      ~factor(preprocess(.x), levels=all_subjid)))
-    })
-  
-  a
-}
+#' load_database(db)
+#' data1$common = data2$common = "Common"
+#' x = enrol %>% 
+#'   edc_left_join(data2) %>% 
+#'   edc_right_join(data1)
+#'   
+#' #crfname get a suffix, common 
+#' names(x)
+edc_left_join = .edc_join(type="left")
+
+#' @rdname edc_left_join
+#' @usage NULL
+#' @export
+edc_right_join = .edc_join(type="right")
+
+#' @rdname edc_left_join
+#' @usage NULL
+#' @export
+edc_full_join = .edc_join(type="full")
 
 
 # Manual correction ---------------------------------------------------------------------------
@@ -388,6 +319,7 @@ harmonize_subjid = function(datalist, preprocess=NULL,
 #' Manual correction
 #' 
 #' @description
+#' `r lifecycle::badge("experimental")`
 #'  
 #' When finding wrong or unexpected values in an exported dataset, it can be useful to temporarily correct them by hard-coding a value. 
 #' However, this manual correction should be undone as soon as the central database is updated with the correction. 
@@ -503,7 +435,7 @@ get_datasets = function(lookup=edc_lookup(), envir=parent.frame()){
     mget(envir=envir, ifnotfound=list(NULL), mode="list", inherits=TRUE)
   a = rtn %>% keep(is.null) %>% names()
   if(length(a) > 5){
-    cli_warn(c("Could not find {length(a)}/{length(rtn)} datasets from the lookup, did you forget to call {.fn load_list} on your import?",
+    cli_warn(c("Could not find {length(a)}/{length(rtn)} datasets from the lookup, did you forget to call {.fn load_database} on your import?",
                i="{.val {a}}"))
   }
   
@@ -525,12 +457,13 @@ get_datasets = function(lookup=edc_lookup(), envir=parent.frame()){
 #'
 #' @return a list(2) of characters with names `patient_id` and `crfname`
 #' 
-#' @export
 #' @importFrom cli cli_warn
 #' @importFrom dplyr lst mutate select
 #' @importFrom lifecycle deprecate_warn
 #' @importFrom purrr map map_chr
 #' @importFrom stats na.omit
+#' @noRd
+#' @keywords internal
 get_key_cols = function(lookup=edc_lookup()){
   patient_id = get_subjid_cols()
   
@@ -574,26 +507,26 @@ get_key_cols = function(lookup=edc_lookup()){
 #' 
 #' @section options:
 #' Use `edc_options()` to set default values:
-#' * `edc_cols_subjid` defaults to `c("PTNO", "SUBJID")`  
-#' * `edc_cols_crfname` defaults to `c("CRFNAME")`
+#' * `edc_cols_subjid` defaults to `c("SUBJID", "PTNO")`  
+#' * `edc_cols_crfname` defaults to `c("FORMDESC", "CRFNAME")`
 #'
 #' @param lookup the lookup table
 #'
 #' @return a character vector
-#' @export
+#' @keywords internal
 #'
 #' @examples
-#' get_subjid_cols()
-#' get_crfname_cols()
+#' #get_subjid_cols()
+#' #get_crfname_cols()
 get_subjid_cols = function(lookup=edc_lookup()){
-  subjid_cols=getOption("edc_cols_subjid", c("PTNO", "SUBJID"))
+  subjid_cols=getOption("edc_cols_subjid", c("SUBJID", "PTNO"))
   .get_key_cols(subjid_cols, id_name="patient", lookup)
 }
 
-#' @rdname get_subjid_cols
-#' @export
+#' @noRd
+#' @keywords internal
 get_crfname_cols = function(lookup=edc_lookup()){
-  crfname_cols=getOption("edc_cols_crfname", "CRFNAME")
+  crfname_cols=getOption("edc_cols_crfname", c("FORMDESC", "CRFNAME"))
   .get_key_cols(crfname_cols, id_name="CRF", lookup)
 }
 
@@ -605,18 +538,21 @@ get_crfname_cols = function(lookup=edc_lookup()){
 .get_key_cols = function(x, id_name, lookup){
   if(is.null(lookup)) return(x)
   
-  f = function(x, y) x[tolower(x) %in% tolower(y)][1] %0% NA 
-  
-  x = map_chr(lookup$names, ~f(.x, x))
+  f = function(x, y) {
+    a = y[tolower(y) %in% tolower(x)][1] %0% NA 
+    if(!a %in% x) a = tolower(a)
+    a
+  }
+  rtn = map_chr(lookup$names, ~f(.x, x))
   
   verbose = getOption("edc_get_key_cols_verbose", FALSE)
-  if(verbose && any(is.na(x))){
+  if(verbose && any(is.na(rtn))){
     cli_warn(c("Default {id_name} identificator could not be found in some datasets", 
-               i='Dataset{?s} without identificator: {rtn[is.na(x), "dataset"]}', 
+               i='Dataset{?s} without identificator: {rtn[is.na(rtn), "dataset"]}', 
                i='Use {.run options(edc_cols_id=c("my_id_col", "my_other_id_col"))}'), 
              class="edcimport_get_key_cols_missing")
   }
-  x %>% na.omit() %>% unique() 
+  rtn %>% na.omit() %>% unique()
 }
 
 
@@ -630,17 +566,18 @@ get_crfname_cols = function(lookup=edc_lookup()){
 #' @param min_pct Default=`0.95`. The minimal proportion of datasets a column has to reach. Subject ID is always excluded.
 #'
 #' @return a character vector
-#' @export
+#' @noRd
+#' @keywords internal
 #'
 #' @examples
-#' tm = edc_example_mixed()
-#' load_list(tm)
+#' db = edc_example()
+#' load_database(db)
 #' meta_cols = get_meta_cols()
 #' long_mixed %>% dplyr::select(-dplyr::any_of(meta_cols))
 #' @importFrom dplyr filter pull setdiff
-get_meta_cols = function(min_pct = getOption("edc_meta_cols_pct", 0.95)){
-  a = get_common_cols(min_datasets=0)
-  subjid_cols = get_subjid_cols()
+get_meta_cols = function(lookup=edc_lookup(), min_pct = getOption("edc_meta_cols_pct", 0.95)){
+  a = get_common_cols(lookup=edc_lookup(), min_datasets=0)
+  subjid_cols = get_subjid_cols(lookup=edc_lookup())
   a %>% filter(pct_datasets>min_pct) %>% pull(column) %>% setdiff(subjid_cols)
 }
 
@@ -664,8 +601,8 @@ get_meta_cols = function(min_pct = getOption("edc_meta_cols_pct", 0.95)){
 #' @importFrom tibble tibble
 #'
 #' @examples
-#' tm = edc_example()
-#' load_list(tm)
+#' db = edc_example()
+#' load_database(db)
 #' x = get_common_cols(min_datasets=1)
 #' x
 #' summary(x)
@@ -680,7 +617,6 @@ get_common_cols = function(lookup=edc_lookup(), min_datasets=3){
       pct_datasets = mean(.data$name_in),
       datasets_in = toString(names(.data$name_in[.data$name_in])),
       datasets_out = toString(names(.data$name_in[!.data$name_in])),
-      # aaa = browser(),
     ) %>% 
     ungroup() %>% 
     arrange(desc(.data$pct_datasets)) %>% 
@@ -713,37 +649,35 @@ summary.common_cols = function(object, ...){
 
 #' Load a list in an environment
 #'
-#' @param x a list
+#' @param db an [edc_database] object (to be fair, any list would do)
 #' @param env the environment onto which the list should be loaded
-#' @param remove if `TRUE`, `x` will be removed from the environment afterward
+#' @param remove if `TRUE`, `db` will be removed from the environment afterward
 #'
 #' @return nothing, called for its side-effect
 #' @export
-#'
-#' @examples
-#' 
-#' x=list(a=1, b=mtcars)
-#' load_list(x, remove=FALSE)
-#' print(a)
-#' print(nrow(b))
-#' 
 #' @importFrom cli cli_abort cli_warn
 #' @importFrom rlang caller_arg
-load_list = function(x, env=parent.frame(), remove=TRUE){
-  if(length(x)==0){
+#'
+#' @examples
+#' db = edc_example()
+#' load_database(db, remove=FALSE)
+#' print(db)
+#' print(lengths(db))
+load_database = function(db, env=parent.frame(), remove=TRUE){
+  if(length(db)==0){
     cli_warn("List was empty.")
     return(invisible())
   }
-  nz = nzchar(names(x))
+  nz = nzchar(names(db))
   if(any(!nz)){
-    cli_abort(c("Every member of {.arg x} should have a name.", 
-                i="Unnamed member{?s} ind{?ex/ices} of {.arg x}: {as.character(which(!nz))}"), 
-              class="load_list_unnamed_error")
+    cli_abort(c("Every member of {.arg db} should have a name.", 
+                i="Unnamed member{?s} ind{?ex/ices} of {.arg db}: {as.character(which(!nz))}"), 
+              class="load_database_unnamed_error")
   }
-  list2env(x, env)
+  list2env(db, env)
   
   if(remove) {
-    x_name = caller_arg(x)
+    x_name = caller_arg(db)
     if(exists(x_name, where=env, inherits=FALSE)) 
       remove(list=x_name, envir=env)
     else                          
@@ -751,33 +685,26 @@ load_list = function(x, env=parent.frame(), remove=TRUE){
   }
 }
 
-#' Load a `.RData` file as a list
-#' 
-#' Instead of loading a `.RData` file in the global environment, extract every object into a list.
-#'
-#' @param filename the filename, with the `.RData` extension.
-#'
-#' @return a list
+
+#' @rdname load_database
+#' @usage NULL
 #' @export
-#'
-#' @examples 
-#' x = list(a=1, b=mtcars)
-#' save_list(x, "test.RData")
-#' y = load_as_list("test.RData")
-#' print(y$a)
-load_as_list = function(filename){
-  temp_env = new.env()
-  load(filename, temp_env)
-  as.list(temp_env)
+#' @importFrom lifecycle deprecate_warn
+load_list = function(db, env=parent.frame(), remove=TRUE){
+  deprecate_warn(when="0.6.0", what="load_list()", with="load_database()")
+  load_database(db, env, remove)
 }
 
+
+
 #' Save a list as `.RData` file
 #'
 #' @param x a list
 #' @param filename the filename, with the `.RData` extension.
 #'
 #' @return nothing, called for its side-effect
-#' @export
+#' @noRd
+#' @keywords internal
 #'
 #' @examples
 #' x=list(a=1, b=mtcars)
@@ -803,13 +730,16 @@ save_list = function(x, filename){
 #' @export
 #' @importFrom cli cat_rule cli_bullets cli_vec
 #' @importFrom purrr discard_at keep
-print.tm_database = function(x, ...){
+print.edc_database = function(x, ...){
   x = x %>% keep(is.data.frame) %>% discard_at(".lookup")
-  cat_rule("Trialmaster database", col = "violet")
+  cat_rule("EDCimport database", col = "violet")
   nms = cli_vec(names(x), list("vec-trunc"=3))
   cli_bullets(c(
-    "{length(x)} tables: {.arg {nms}}",
-    i="Use {.code EDCimport::load_list(tm)} to load the tables in the global environment",
-    i="Use {.code print(tm$.lookup)} to see the summary table"
+    "Contains {length(x)} table{?s}: {.arg {nms}}",
+    i="Use {.run EDCimport::load_database(db)} to load the tables in the global environment.",
+    i="Use {.run EDCimport::edc_lookup()} to see the summary table."
   ))
 }
+
+#' @export
+print.tm_database = print.edc_database
