@@ -20,14 +20,14 @@
 #' @export
 #'
 #' @examples
-#' tm = edc_example()
-#' load_list(tm)
-#' options(edc_subjid_ref=db0$SUBJID)
+#' db = edc_example()
+#' load_database(db)
+#' options(edc_subjid_ref=enrol$subjid)
 #' #usually, you set something like:
 #' #options(edc_subjid_ref=enrolres$subjid)
-#' edc_warn_patient_diffs(db1)
-#' db1 %>% dplyr::filter(SUBJID>1) %>% edc_warn_patient_diffs()
-#' edc_warn_patient_diffs(c(db1$SUBJID, 99, 999))
+#' edc_warn_patient_diffs(data1)
+#' data1 %>% dplyr::filter(subjid>1) %>% edc_warn_patient_diffs(issue_n=NULL)
+#' edc_warn_patient_diffs(c(data1$subjid, 99, 999))
 edc_warn_patient_diffs = function(x, ref=getOption("edc_subjid_ref"), 
                                   issue_n="xx", data_name=NULL, 
                                   col_subjid=get_subjid_cols()){
@@ -93,8 +93,8 @@ edc_warn_patient_diffs = function(x, ref=getOption("edc_subjid_ref"),
 #' @export
 #'
 #' @examples
-#' tm = edc_example()
-#' load_list(tm)
+#' db = edc_example()
+#' load_database(db)
 #' edc_warn_extraction_date()
 #' @importFrom cli cli_warn
 edc_warn_extraction_date = function(max_days=30){
@@ -171,17 +171,17 @@ assert_no_duplicate = function(df, by=NULL, id_col=get_subjid_cols()){
 #' 
 #' When checking your data, filter your dataset to get only problematic rows. \cr
 #' Then, use either:
-#'  * `edc_data_warn()` to generate a standardized warning that can be forwarded to the datamanager 
-#'  * `edc_data_warn()` to abort the script if the problem is too serious
+#'  * `edc_data_warn()` to generate a standardized warning that can be forwarded to the datamanager.
+#'  * `edc_data_stop()` to abort the script if the problem is too serious.
 #'  
-#' Database issues should be traced in a separate file, each with an identifying row number, and the file should be shared with the data-manager. \cr
-#' Use `edc_data_warnings()` to generate the table for such a file.
+#' Each time [edc_data_warn] is used, the warning is saved internally so that a summary of all your warnings can be retrieved using [edc_data_warnings]. \cr The result can be saved into an Excel file using [save_edc_data_warnings()].
 #'
 #' @param df the filtered dataframe
 #' @param message the message. Can use [cli formats](https://cli.r-lib.org/reference/inline-markup.html#classes). `df` can be accessed using the `.data` special keyword (see example)
 #' @param issue_n identifying row number
 #' @param csv_path a path to save `df` in a csv file that can be shared with the DM for more details.
 #' @param max_subjid max number of subject ID to show in the message
+#' @param envir the environment to evaluate `message` in.
 #' @param col_subjid column name for subject ID. Set to `NULL` to ignore.
 #' @param ... unused 
 #'
@@ -191,60 +191,62 @@ assert_no_duplicate = function(df, by=NULL, id_col=get_subjid_cols()){
 #'
 #' @examples
 #' library(dplyr)
-#' tm = edc_example()
-#' load_list(tm)
-#' db0 %>% 
+#' db = edc_example()
+#' load_database(db)
+#' enrol %>% 
 #'   filter(age>70) %>% 
 #'   edc_data_warn("Age should not be >70", issue_n=1)
 #' 
-#' db0 %>% 
+#' enrol %>% 
 #'   filter(age<25) %>% 
 #'   edc_data_warn("Age should not be <25", issue_n=2)
 #' 
-#' db1 %>% 
-#'   filter(n()>1, .by=SUBJID) %>% 
-#'   edc_data_warn("There are duplicated patients in `db1` ({nrow(.data)} rows)", issue_n=3)
+#' data1 %>% 
+#'   filter(n()>1, .by=subjid) %>% 
+#'   edc_data_warn("There are duplicated patients in `data1` ({nrow(.data)} rows)", issue_n=3)
 #' 
-#' db0 %>% 
+#' enrol %>% 
 #'   filter(age<25) %>% 
 #'   edc_data_warn("Age should not be <25", issue_n=NULL)
 #'   
 #' edc_data_warnings()
 #' 
 #' \dontrun{
-#' db0 %>% 
+#' enrol %>% 
 #'   filter(age<25) %>% 
 #'   edc_data_warn("Age should not be <25", csv_path="check/check_age_25.csv")
 #'   
-#' db0 %>% 
+#' enrol %>% 
 #'   filter(age<25) %>% 
 #'   edc_data_stop("Age should *never* be <25")
 #' }
 edc_data_warn = function(df, message, ..., 
                          issue_n="xx", max_subjid=5, 
                          csv_path=FALSE, 
+                         envir=parent.frame(), 
                          col_subjid=get_subjid_cols()){
   
   if (missing(max_subjid)) max_subjid = getOption("edc_warn_max_subjid", max_subjid)
   check_dots_empty()
-  edc_data_condition(.data=df, message=message, issue_n=issue_n, csv_path=csv_path,
-                     max_subjid=max_subjid, col_subjid=col_subjid, 
+  .edc_data_condition(tbl=df, message=message, issue_n=issue_n, csv_path=csv_path,
+                     max_subjid=max_subjid, .envir=envir, col_subjid=col_subjid, 
                      fun=cli_warn)
 }
 
 #' @rdname edc_data_warn
-#' @usage edc_data_stop(df, message, ..., issue_n, max_subjid, csv_path, col_subjid)
+#' @usage edc_data_stop(df, message, ..., issue_n, max_subjid, csv_path, envir, col_subjid)
 #' @export
 #' @importFrom rlang check_dots_empty
 edc_data_stop = function(df, message, ..., 
                          issue_n="xx", max_subjid=5, 
                          csv_path=FALSE, 
+                         envir=parent.frame(), 
                          col_subjid=get_subjid_cols()){
   
   if (missing(max_subjid)) max_subjid = getOption("edc_warn_max_subjid", max_subjid)
   check_dots_empty()
-  edc_data_condition(.data=df, message=message, issue_n=issue_n, csv_path=csv_path,
-                     max_subjid=max_subjid, col_subjid=col_subjid, 
+  .edc_data_condition(tbl=df, message=message, issue_n=issue_n, csv_path=csv_path,
+                     max_subjid=max_subjid, .envir=envir, col_subjid=col_subjid, 
                      fun=cli_abort)
 }
 
@@ -259,7 +261,50 @@ edc_data_stop = function(df, message, ...,
 edc_data_warnings = function(){
   edcimport_env$warn_list %>% 
     list_rbind() %>% 
-    arrange(across(any_of(c("issue_n", "message"))))
+    arrange(across(any_of(c("issue_n", "message")))) %>% 
+    add_class("edc_warning_summary")
+}
+
+
+
+#' Save EDCimport warning to Excel
+#' 
+#' Each time [edc_data_warn] is used, the warning is saved internally so that a summary can be retrieved using [edc_data_warnings]. This summary can then be saved into a `.xlsx` file using `save_edc_data_warnings()`. 
+#'
+#' @param edc_warnings the result of [edc_data_warnings]
+#' @param path a `.xlsx` file path
+#' @param overwrite If `TRUE`, overwrite any existing file.
+#' @param open If `TRUE`, overwrite any existing file.
+#'
+#' @returns a logical(1), whether the file could be written, invisibly 
+#' @export
+save_edc_data_warnings = function(edc_warnings=edc_data_warnings(), 
+                                  path="edc_data_warnings.xlsx",
+                                  overwrite=TRUE, 
+                                  open=FALSE){
+  check_installed("openxlsx", reason="for `save_edc_data_warnings()` to work.")
+  assert_class(edc_warnings, "edc_warning_summary", null.ok=FALSE)
+  assert(path_ext(path)=="xlsx")
+  
+  edc_warnings$issue_n %>% make.unique
+  
+  wb = openxlsx::createWorkbook()
+  for(i in seq(nrow(edc_warnings))){
+    if(!exists("date_extraction")) date_extraction=NULL
+    x = edc_warnings[i, ]
+    sheet = paste0("issue_", x$issue_n)
+    data = x$data[[1]]
+    color = if(nrow(data)==0) "green" else "red"
+    openxlsx::addWorksheet(wb, sheet, tabColour=color, gridLines=FALSE)
+    openxlsx::writeData(wb, sheet, tibble(date_extraction, x$message), colNames=FALSE)
+    openxlsx::writeDataTable(wb, sheet, data, startRow=2)
+  }
+  rtn = openxlsx::saveWorkbook(wb, path, overwrite=overwrite, returnValue=TRUE)
+  
+  if(isTRUE(open) && isTRUE(rtn)){
+    browseURL(path)
+  }
+  invisible(rtn)
 }
 
 
@@ -270,72 +315,99 @@ edc_data_warnings = function(){
 #' @keywords internal
 #' @importFrom cli cli_abort cli_vec cli_warn format_inline
 #' @importFrom dplyr pull
-#' @importFrom rlang caller_arg
+#' @importFrom fs dir_create
+#' @importFrom rlang caller_arg env_bind
 #' @importFrom stringr str_ends str_pad
 #' @importFrom tibble tibble
 #' @importFrom utils write.csv2
-edc_data_condition = function(.data, message, issue_n, max_subjid, 
-                              csv_path, 
+.edc_data_condition = function(tbl, message, issue_n, max_subjid, 
+                              csv_path, .envir, 
                               col_subjid, fun){
-  if(nrow(.data)>0){
-    if(is.character(csv_path)){
-      assert(str_ends(csv_path, "\\.csv"), call=parent.frame())
-      write.csv2(.data, csv_path, row.names=FALSE)
-    }
-    message = format_inline(message)
-    
-    par_issue = par_subj = ""; subj=NULL
-    if(is.null(issue_n)) issue_n = NA
-    
-    if(!is.na(issue_n)){
-      if(is.numeric(issue_n)) issue_n = str_pad(issue_n, width=2, pad="0")
-      par_issue = format_inline("Issue #{col_green(issue_n)}: ")
-    }
-    
-    if(!is.null(col_subjid)){
-      col_found = tolower(col_subjid) %in% tolower(names(.data))
-      if(sum(col_found)>1){
-        cli_warn("Found {length(col_found)} subject identifiers in the input dataset:
-                 {.val {col_subjid[col_found]}}. Defaulting to the first one.", 
-                  class="edc_data_condition_subjid_multiple_warn", call=parent.frame())
-        col_subjid = col_subjid[col_found][1]
-      }
-      if(!any(col_found)){
-        cli_abort("Could not find column {col_subjid} in the input dataset.", 
-                  class="edc_data_condition_subjid_error", call=parent.frame())
-      }
-      subj = .data %>% pull(any_of2(col_subjid)) %>% unique() %>% sort()
-      n_subj = length(subj)
-      subj = paste0("#", subj) %>% 
-        cli_vec(style=list("vec_trunc"=max_subjid, "vec-trunc-style"="head"))
-      par_subj = format_inline(" ({n_subj} patient{?s}: {subj})")
-    }
-    fun_name = caller_arg(fun)
-    item = tibble(issue_n, message, subjid=list(subj), fun=fun_name)
-    save_warn_list_item(item)
-    
-    fun("{par_issue}{message}{par_subj}")
+  if(is.character(csv_path)){
+    assert(str_ends(csv_path, "\\.csv"), call=parent.frame())
+    dir_create(dirname(csv_path))
+    write.csv2(tbl, csv_path, row.names=FALSE)
   }
-  invisible(.data)
+  
+  env_bind(.envir, .data=tbl)
+  message = format_inline(message, .envir=.envir)
+  
+  par_issue = par_subj = ""; subj=NULL
+  if(is.null(issue_n)) issue_n = NA
+  
+  if(!is.na(issue_n)){
+    if(is.numeric(issue_n)) issue_n = str_pad(issue_n, width=2, pad="0")
+    par_issue = format_inline("Issue #{col_green(issue_n)}: ")
+  }
+  
+  if(!is.null(col_subjid)){
+    col_found = tolower(col_subjid) %in% tolower(names(tbl))
+    if(sum(col_found)>1){
+      cli_warn("Found {length(col_found)} subject identifiers in the input dataset:
+                 {.val {col_subjid[col_found]}}. Defaulting to the first one.", 
+               class="edc_data_condition_subjid_multiple_warn", call=parent.frame())
+      col_subjid = col_subjid[col_found][1]
+    }
+    if(!any(col_found)){
+      cli_abort("Could not find column {.val {col_subjid}} in the input dataset.", 
+                class="edc_data_condition_subjid_error", call=parent.frame())
+    }
+    subj = tbl %>% pull(any_of2(col_subjid)) %>% unique() %>% sort()
+    n_subj = length(subj)
+    subj = paste0("#", subj) %>% 
+      cli_vec(style=list("vec_trunc"=max_subjid, "vec-trunc-style"="head"))
+    par_subj = format_inline(" ({n_subj} patient{?s}: {subj})")
+  }
+  
+  item_subjid = NULL
+  if(nrow(tbl)>0){
+    fun("{par_issue}{message}{par_subj}", class="edc_data_condition")
+    item_subjid=list(subj)
+  }
+  
+  item = tibble(issue_n, message, subjid=item_subjid, data=list(tbl))
+  save_warn_list_item(item)
+    
+  invisible(tbl)
 }
 
 
 #' @noRd
 #' @keywords internal
 #' @importFrom cli cli_warn
+#' @importFrom rlang has_name
+#' @importFrom dplyr semi_join
 save_warn_list_item = function(item){
   stopifnot(nrow(item)==1)
+  if(is.na(item$issue_n)) return(NULL)
+  VERBOSE = getOption("edc_warn_duplicate_verbose", TRUE)
+  current = edc_data_warnings()
+  
+  if(item$issue_n=="xx") {
+    n_xx = sum(str_detect(current$issue_n, "^xx\\d+$"))
+    item$issue_n = paste0("xx", n_xx+1)
+  }
   issue_n = item$issue_n
   issue_key = paste0("issue_", item$issue_n)
-  current = edc_data_warnings()
-  if(!is.na(item$issue_n) && item$issue_n %in% current$issue_n){
-    if(item$issue_n=="xx" && !item$message %in% current$message){
-      issue_key = paste0("issue_xx_", nrow(current))
-    } else if(getOption("edc_warn_duplicate_verbose", FALSE)){
-      cli_warn("Duplicate `edc_data_warn()` entry")
+  
+  #warn if same issue number
+  if(issue_n %in% current$issue_n && VERBOSE){
+    cli_warn("Duplicated `edc_data_warn()` entry: issue_n={.val {issue_n}}", 
+             class="save_warn_list_item_dupln_warning")
+  }
+  
+  #warn & skip if same message and data
+  if(nrow(current)>0 && all(has_name(item, c("message", "data")))){
+    m = semi_join(item, current, by=c("message", "data"))
+    if(nrow(m)>0 && VERBOSE) {
+      cli_warn("Duplicated `edc_data_warn()` entry: {.val {item$message}}", 
+               class="save_warn_list_item_dupl_warning")
+      return(NULL)
     }
   }
+
   edcimport_env$warn_list[[issue_key]] = item
+  NULL
 }
 
 #' @noRd
