@@ -2,7 +2,7 @@
 
 #' Identify if a dataframe has a long or a wide format
 #' 
-#' A dataset is either in the wide format or in the long format ([link](https://towardsdatascience.com/long-and-wide-formats-in-data-explained-e48d7c9a06cb)). 
+#' A dataset is either in the wide format or in the long format. 
 #' This function identifies the format of a dataframe with respect to a subject ID. 
 #' If a dataframe has some wide and long columns, it is considered "mixed".
 #' 
@@ -10,16 +10,17 @@
 #' @param df a dataframe
 #' @param id the identifying subject ID
 #' @param ... not used
-#' @param ignore_cols columns to ignore. Usually meta columns (see [get_meta_cols]).
+#' @param ignore_cols columns to ignore.
 #' @param na_rm whether to consider missing values
 #' @param warn whether to warn if ID is not found
 #'
 #' @return a string value in `c("wide", "long", "mixed)`
 #' @export
+#' @seealso <https://tidyr.tidyverse.org/articles/pivot.html>
 #' 
 #' @examples
-#' tm = edc_example_mixed()
-#' sapply(tm, table_format, warn=FALSE) 
+#' db = edc_example()
+#' sapply(db, table_format, warn=FALSE) 
 table_format = function(df, id=get_subjid_cols(), ..., 
                         ignore_cols=get_meta_cols(0.95), 
                         na_rm=FALSE,
@@ -65,46 +66,83 @@ table_format = function(df, id=get_subjid_cols(), ...,
   mean_nval
 }
 
-
 #' Split mixed datasets
 #' 
 #' Split mixed tables, i.e. tables that hold both long data (N values per patient) and short data (one value per patient, duplicated on N lines), into one long table and one short table.
 #'
-#' @param datasets a dataframe or a list of dataframes to split. Default to all the datasets from `.lookup`.
-#' @param id the patient identifier, probably "SUBJID". Should be shared by all datasets. Case-insensitive.
-#' @param ignore_cols columns to ignore when considering a table as long. Default to `getOption("edc_cols_crfname", "CRFNAME")`. Case-insensitive.
-#' @param output_code whether to print the code to explicitly write. Can also be a file path.
+#' @param database an [edc_database] object, from [read_trialmaster()] or other EDCimport reading functions.
+#' @param datasets <[tidy-select][dplyr::dplyr_tidy_select]> datasets to split in the database
+#' @param ignore_cols columns to ignore in long tables. Default to `getOption("edc_cols_crfname", "CRFNAME")`. Case-insensitive. Avoid splitting tables for useless columns.
 #' @param verbose whether to print informations about the process.
-#' @param ... not used
+#' @param ... not used, ensure arguments are named
 #'
-#' @return a list of the new long and short tables. Use [load_list()] to load them into the global environment.
+#' @return an [edc_database] object
 #' @export
-#' @importFrom cli cli_bullets cli_warn
-#' @importFrom dplyr across all_of everything group_by lst select summarise summarise_all ungroup
-#' @importFrom glue glue
-#' @importFrom purrr discard discard_at imap keep list_flatten map_chr
-#' @importFrom rlang check_dots_empty
-#' @importFrom utils head
+#' @importFrom cli cli_abort
+#' @importFrom dplyr enquo everything
+#' @importFrom purrr discard_at keep
+#' @importFrom rlang as_label check_dots_empty
 #'
 #' @examples
-#' #tm = read_trialmaster("filename.zip", pw="xx")
-#' tm = edc_example_mixed()
-#' names(tm)
-#' #load_list(tm)
-#' print(tm$long_mixed) #`val1` and `val2` are long but `val3` is short
+#' #db = read_trialmaster("filename.zip", pw="xx")
+#' db = edc_example() %>% 
+#'   edc_split_mixed(c(ae, starts_with("long")), 
+#'                   ignore_cols="crfstat")
+#'   
+#' names(db)
+#' edc_lookup()
 #' 
-#' mixed_data = split_mixed_datasets(tm, id="subjid", verbose=TRUE)
-#' load_list(mixed_data)
-#' print(long_mixed_short) 
-#' print(long_mixed_long) 
+#' db$ae #`aesoc`, `aegr`, and `sae` are long, but `n_ae` is short
 #' 
-#' #alternatively, get the code and only use the datasets you need
-#' split_mixed_datasets(tm, id="SUBJID", output_code=TRUE)
-#' filename = tempfile("mixed_code", fileext=".R")
-#' split_mixed_datasets(tm, id="SUBJID", output_code=filename)
-#' readLines(filename)
+#' db$ae_short
+#' db$ae_long
+edc_split_mixed = function(database, datasets=everything(), 
+                           ...,
+                           ignore_cols=NULL, 
+                           verbose=FALSE){
+  check_dots_empty()
+  .lookup = database$.lookup
+  patient_id = get_subjid_cols(lookup=.lookup)
+  
+  ignore_cols_default = c(get_crfname_cols(lookup=.lookup), 
+                          get_meta_cols(lookup=.lookup, min_pct=0.95))
+  if(!is.null(ignore_cols)){
+    ignore_cols = c(ignore_cols_default, ignore_cols)
+  }
+  
+  db_mixed = database %>% 
+    keep(is.data.frame) %>% 
+    discard_at(".lookup") %>% 
+    list_select({{datasets}}) %>% 
+    split_mixed_datasets(id=patient_id, verbose=verbose, 
+                         ignore_cols=ignore_cols, output_code=FALSE)
+  
+  if(length(db_mixed)==0){
+    split_mixed_names = as_label(enquo(datasets))
+    cli_abort("Dataset{?s} {.val {split_mixed_names}} are not mixed 
+                 (either short or long) and cannot be splitted.", 
+             class="edc_read_cannot_split_mixed_warn")
+  }
+  database = c(database, db_mixed)
+
+  database$.lookup = database %>% build_lookup() %>% extend_lookup(datasets=database)
+  .set_lookup(database$.lookup, verbose=FALSE)
+  
+  class(database) = "edc_database"
+  
+  database
+}
+
+#' @noRd
+#' @keywords internal
+#' @importFrom cli cli_abort cli_bullets cli_warn
+#' @importFrom dplyr across all_of everything group_by lst select summarise summarise_all ungroup
+#' @importFrom glue glue
+#' @importFrom purrr discard discard_at imap keep list_flatten map_chr map_lgl
+#' @importFrom rlang check_dots_empty
+#' @importFrom utils head
 split_mixed_datasets = function(datasets=get_datasets(), id=get_subjid_cols(), ..., 
-                                ignore_cols=get_meta_cols(0.95), 
+                                ignore_cols=get_meta_cols(min_pct=0.95), 
                                 output_code=FALSE,
                                 verbose=TRUE){
   check_dots_empty()
@@ -112,6 +150,13 @@ split_mixed_datasets = function(datasets=get_datasets(), id=get_subjid_cols(), .
   override_ignore_cols = options("edc_override_ignore_cols")[[1]]
   if(!is.null(override_ignore_cols)) ignore_cols = override_ignore_cols
   datasets = datasets %>% keep(is.data.frame) %>% discard_at(".lookup")
+
+  id_found = map_lgl(datasets, ~any(tolower(id) %in% tolower(names(.x))))
+  if(!any(id_found)){
+    cli_abort("Patient ID column {.val {id}} was not found in any of 
+              {.arg datasets} ( {.val {names(mixed)}}).",
+              class="edc_subjid_not_found")
+  }
   
   dataset_mean_nval = datasets %>%
     imap( ~ {
@@ -135,7 +180,6 @@ split_mixed_datasets = function(datasets=get_datasets(), id=get_subjid_cols(), .
   #TODO option pour faire plutôt length(unique(na.omit(.x))) ?
   #si c'est manquant sur une ligne et pas sur une autre on 
   #peut sans doute unifier quand même
-  
   not_found = dataset_mean_nval %>% keep(is.null) %>% names()
   if(length(not_found) == length(datasets)){
     cli_warn("{.val {id}} was not found in any table. Returning {.val NULL}")
@@ -226,7 +270,7 @@ split_mixed_datasets = function(datasets=get_datasets(), id=get_subjid_cols(), .
                   " "="{.run utils::browseURL({output_code})}"))
     cat(code, file=output_code, append=TRUE)
   } else if(verbose){
-    cli_bullets(c(">"="Use {.fun EDCimport::load_list} on the result to get separated long and short data."))
+    cli_bullets(c(">"="Use {.fun EDCimport::load_database} on the result to get separated long and short data."))
   }
   
   rtn %>% 
@@ -240,9 +284,14 @@ split_mixed_datasets = function(datasets=get_datasets(), id=get_subjid_cols(), .
 
 #' Unify a vector
 #' 
-#' Turn a vector of length N to a vector of length 1 after checking that there is only one unique value. Useful to safely flatten a duplicated table. This preserves the `label` attribute if set.
+#' Turn a vector of length N to a vector of length 1 after checking that there 
+#' is only one unique value. 
+#' Useful to safely flatten a duplicated table. 
+#' Preserves the `label` attribute if set.
 #'
 #' @param x a vector
+#' @param collapse_chr whether to collapse non-unique character values 
+#' @param warn whether to warn if non-unique values were found
 #'
 #' @return a vector of length 1
 #' @export
@@ -254,18 +303,32 @@ split_mixed_datasets = function(datasets=get_datasets(), id=get_subjid_cols(), .
 #' #unify(c(1,1,2,1)) #warning
 #' 
 #' library(dplyr)
-#' x=tibble(id=rep(letters[1:5],10), value=rep(1:5,10))
-#' x %>% group_by(id) %>% summarise(value=unify(value)) #safer than `value=value[1]`
+#' set.seed(42)
+#' x=tibble(id=rep(letters[1:5],10), value=rep(1:5,10), 
+#'          value2=sample(letters[6:10], 50, replace=TRUE))
+#' x %>% summarise(value=unify(value), .by=id) #safer than `value=value[1]`
+#' x %>% summarise(value2=unify(value2, collapse_chr=TRUE, warn=FALSE), .by=id)
 #' x$value[2]=1
-#' #x %>% group_by(id) %>% summarise(value=unify(value)) #warning about that non-unique value
-unify = function(x){
+#' x %>% summarise(value2=unify(value2), .by=id) #warning about that non-unique value
+unify = function (x, collapse_chr=FALSE, warn=TRUE) {
   rtn = x[1]
   lu = length(unique(na.omit(x)))
-  if(lu>1){
-    cli_warn(c("Unifying multiple values in {.val {caller_arg(x)}}, returning the first one ({.val {rtn})}", 
-               i="Unique values: {.val {unique(na.omit(x))}}"))
+  if (lu > 1) {
+    if(isTRUE(collapse_chr)){
+      if(is.character(x) || is.factor(x)){
+        if(isTRUE(warn)){
+          cli_warn(c("Collapsed multiple values in {.val {caller_arg(x)}}",
+                     i = "Unique values: {.val {unique(na.omit(x))}}"))
+        }
+        rtn = toString(x)
+      }
+    } else if(isTRUE(warn)){
+      cli_warn(c("Unifying multiple values in {.val {caller_arg(x)}}, returning the first one ({.val {rtn})}",
+                 i = "Unique values: {.val {unique(na.omit(x))}}"))
+    }
   }
   rtn_label = get_label(x)
-  if(!is.null(rtn_label)) attr(rtn, "label") = rtn_label
+  if (!is.null(rtn_label))
+    attr(rtn, "label") = rtn_label
   rtn
 }
