@@ -5,7 +5,7 @@ edc_options(edc_lookup_overwrite_warn=FALSE)
 test_that("Read a TM archive", {
   clean_cache()
   clean_lookup()
-  local_reproducible_output(width=125)
+  local_reproducible_output(width=125, unicode=TRUE)
   
   f = function(x) {
     x %>% 
