@@ -18,45 +18,35 @@ test_that("Read TM with cache", {
   clean_lookup()
   edc_options(edc_lookup_overwrite_warn=TRUE, .local=TRUE)
   
+  #first read, read from zip (default: use_cache=write)
   w = read_trialmaster(filename) %>% 
-    expect_classed_conditions(message_class="read_tm_zip")
+    expect_classed_conditions(message_class=c("read_tm_zip", "edc_create_cache"))
   
+  #2nd, use_cache=TRUE -> read from cache
   w = read_trialmaster(filename, use_cache=TRUE) %>%
     expect_classed_conditions(message_class="read_tm_cache",
                               warning_class="edc_lookup_overwrite_warn")
-  # if(FALSE){
-    
-
-
+  
+  #3rd, use_cache=write -> read from zip again
   w = read_trialmaster(filename, use_cache="write") %>%
-    expect_classed_conditions(message_class="read_tm_zip",
+    expect_classed_conditions(message_class=c("read_tm_zip", "edc_create_cache"),
                               warning_class="edc_lookup_overwrite_warn")
+  
+  #4th, use_cache=read -> read from cache again
   w = read_trialmaster(filename, use_cache="read") %>%
     expect_classed_conditions(message_class="read_tm_cache",
                               warning_class="edc_lookup_overwrite_warn")
-  # expect_message(w <- read_trialmaster(filename),
-  #                class="read_tm_zip")
-  # expect_message(w <- read_trialmaster(filename),
-  #                class="read_tm_cache")
-  # expect_message(w <- read_trialmaster(filename, use_cache=FALSE),
-  #                class="read_tm_zip")
-  # expect_warning(w <- read_trialmaster(filename, use_cache=FALSE),
-  #                class="edc_lookup_overwrite_warn")
-
+  
+  #5th, use_cache=F -> read from zip again again
   w = read_trialmaster(filename, use_cache=FALSE) %>%
     expect_classed_conditions(message_class="read_tm_zip",
                               warning_class="edc_lookup_overwrite_warn")
-  # w = read_trialmaster(filename, use_cache=FALSE) %>% 
-  #   expect_classed_conditions(message_class=c("read_tm_zip", "xxxx"),
-  #                             warning_class="edc_lookup_overwrite_warn")
-  # }
   
-  # class(es[[1]])
-  # [1] "expectation_failure" "expectation"         "error"               "condition"      
+  
   expect_length(w, 8)
 
 
-  load_list(w, remove=FALSE)
+  load_database(w, remove=FALSE)
   expect_true(exists("pat"))
   expect_length(pat, 35)
   expect_true(nrow(trial)==1)
@@ -67,7 +57,7 @@ test_that("Read TM with cache", {
   #expect formats
   expect_s3_class(site$INCLSITE, "factor")
   expect_equal(as.character(site$INCLSITE), "Yes")
-  expect_equal(dim(.lookup), c(5,8))
+  expect_equal(dim(.lookup), c(5,9))
   clean_cache()
 })
 
@@ -82,14 +72,27 @@ test_that("Read TM without procformat", {
 
 
 test_that("Read TM with a bad name", {
-  expect_warning(w <- read_trialmaster(filename_bad, use_cache=FALSE, verbose=0),
-                 class="edc_tm_bad_name")
+  w = read_trialmaster(filename_bad, use_cache=FALSE, verbose=0) %>% 
+    expect_classed_conditions(warning_class=c("edc_tm_bad_name", 
+                                              "get_folder_datetime_warning"))
   expect_false(is.na(w$datetime_extraction))
   expect_false(is.na(w$date_extraction))
   expect_equal(as.character(w$site$INCLSITE), "Yes")
   clean_cache()
 })
 
+test_that("Read TM with an internal structure", {
+  local_options(edc_lookup_overwrite_warn=FALSE)
+  tm = read_trialmaster(test_path("CRF_Dan_Export_SAS_XPORT_2022_08_25_15_16_subdir.zip"),
+                        use_cache=FALSE, verbose=0, subdirectories=FALSE)
+  tm2 = read_trialmaster(test_path("CRF_Dan_Export_SAS_XPORT_2022_08_25_15_16_subdir.zip"),
+                         use_cache=FALSE, verbose=0, subdirectories=TRUE)
+  expect_contains(names(tm), c("vs", "visit"))
+  expect_contains(names(tm2), c("vs", "visit", "sub_vs"))
+  expect_false(any(names(tm) == "sub_vs"))
+  clean_cache()
+})
+
 
 test_that("Use cache only if permitted", {
   clean_cache()
@@ -99,7 +102,8 @@ test_that("Use cache only if permitted", {
   f = function(df) dplyr::rename_with(df, tolower)
   clean_lookup()
   w2 = read_trialmaster(filename, use_cache="read", verbose=0) %>% expect_silent()
-  w2 = read_trialmaster(filename, use_cache="read", verbose=0, clean_names_fun=f, split_mixed=TRUE) %>% 
+  w2 = read_trialmaster(filename, use_cache="read", verbose=0, 
+                        clean_names_fun=f) %>% 
     expect_error(class="read_tm_cache_bad_param")
   clean_cache()
 })
