@@ -13,28 +13,28 @@ test_that("no exports", {
 
 
 
-# load_list() ---------------------------------------------------------------------------------
+# load_database() ---------------------------------------------------------------------------------
 
-test_that("load_list() works", {
+test_that("load_database() works", {
   x=list(a=1, b=mtcars)
-  load_list(x, remove=TRUE)
+  load_database(x, remove=TRUE)
   expect_equal(a,1)
   expect_length(b,11)
   expect_false(exists("x", inherits=FALSE))
 })
 
-test_that("load_list() works without remove", {
+test_that("load_database() works without remove", {
   x=list(a=1, b=mtcars)
-  load_list(x, remove=FALSE)
+  load_database(x, remove=FALSE)
   expect_equal(a,1)
   expect_length(b,11)
   expect_true(exists("x", inherits=FALSE))
 })
 
-test_that("load_list() errors", {
+test_that("load_database() errors", {
   x=list(a=1, 5, b=8, 9)
-  expect_error(load_list(x),
-               class="load_list_unnamed_error")
+  expect_error(load_database(x),
+               class="load_database_unnamed_error")
 })
 
 
@@ -48,28 +48,50 @@ test_that("save_list() works", {
 })
 
 test_that("get_folder_datetime() works", {
+  
+  #good example
+  get_folder_datetime("data/extraction20000101/") %>% 
+    expect_s3_class(c("POSIXt", "Date")) %>% 
+    as.character() %>% 
+    expect_equal("2000-01-01")
+  get_folder_datetime("data/extraction-2000_1~1/") %>% #bad format though
+    expect_s3_class(c("POSIXt", "Date")) %>% 
+    as.character() %>% 
+    expect_equal("2000-01-01")
+  
+  #Warning: multiple dates
+  get_folder_datetime("data/template_2020_01_01_extraction_2000-01-01/") %>% 
+    expect_classed_conditions(warning_class=c("extract_date_multiple_warning")) %>% 
+    expect_s3_class(c("POSIXt", "Date")) %>% 
+    as.character() %>% 
+    expect_equal("2000-01-01")
+  
+  #Warning: defaults to modification date
   folder = paste0(tempdir(), "/test_get_datetime")
   fs::dir_create(folder)
   fs::file_create(paste0(folder, "/f", 1:5, ".R"))
-  dir(folder, full.names=TRUE)[1:3] %>% purrr::walk(~Sys.setFileTime(.x, "1975-01-01 CET"))
-  # dir(folder, full.names=TRUE) %>% file.info() %>% select(mtime)
-  x = get_folder_datetime(folder) %>% 
-    expect_classed_conditions(warning_class="get_folder_datetime_modiftime_warning")
-  expect_equal(as.character(x), "1975-01-01")
+  dir(folder, full.names=TRUE)[1:3] %>% purrr::walk(~Sys.setFileTime(.x, "2000-01-01 CET"))
+  # dir(folder, full.names=TRUE) %>% file.info() %>% as_tibble(rownames="file")
+  get_folder_datetime(folder) %>% 
+    expect_classed_conditions(warning_class=c("get_folder_datetime_warning", 
+                                              "get_folder_datetime_modiftime_warning")) %>% 
+    expect_s3_class(c("POSIXt", "Date")) %>% 
+    as.character() %>% 
+    expect_equal("2000-01-01")
+  fs::dir_delete(folder)
 })
 
 
-# build_lookup() & find_keyword() ---------------------------------------------------------------
+# build_lookup() & edc_find_column() ---------------------------------------------------------------
 
 test_that("build_lookup() works", {
+  excluded = c(".lookup", "date_extraction", "datetime_extraction")
+  x = edc_example() %>% discard_at(excluded)
+  lookup = build_lookup(x) %>% arrange(dataset)
+  xnames = x[sort(names(x))] %>% map(names)
   
-  x = edc_example()
-  x$.lookup=NULL
-  lookup = build_lookup(x)
-  expect_equal(lengths(lookup$names), c(db0=5,db2=5,db3=6,db1=6))
-  expect_true(all(nzchar(lookup$labels$i)))
-  expect_false(any(nzchar(lookup$labels$m)))
-  # lookup %>% unnest(everything())
+  expect_equal(lookup$names, xnames)
+  expect_true(all(nzchar(unlist(lookup$labels))))
   
   x = list(i=iris, mtcars)
   build_lookup(x) %>% 
@@ -79,23 +101,23 @@ test_that("build_lookup() works", {
     expect_error(class="edc_lookup_empty")
 })
 
-test_that("find_keyword() works", {
+test_that("edc_find_column() works", {
   x = edc_example()
   # x$.lookup %>% unnest() %>% v
-  x1=find_keyword("visit", data=x$.lookup)
+  x1=edc_find_column("visit", lookup=x$.lookup)
   expect_setequal(x1$names, paste0("date", 1:10))
-  x2=find_keyword("id|\\(", data=x$.lookup)
-  expect_equal(unique(x2$names), c("SUBJID", "age"))
-  x3=find_keyword("id|\\(", data=x$.lookup, ignore_case=FALSE)
+  x2=edc_find_column("SUBJ|\\(", lookup=x$.lookup)
+  expect_equal(unique(x2$names), c("subjid", "age"))
+  x3=edc_find_column("SUBJ|\\(", lookup=x$.lookup, ignore_case=FALSE)
   expect_equal(unique(x3$names), "age")
 })
 
 
-test_that("find_keyword() works with read_trialmaster()", {
+test_that("edc_find_column() works with read_trialmaster()", {
   clean_cache()
   w = read_trialmaster(filename, use_cache=FALSE, verbose=0)
   local_options(edc_lookup=w$.lookup)
-  x1=find_keyword("sex")
+  x1=edc_find_column("sex")
   expect_equal(x1$names, "SEX")
 })
 
@@ -108,11 +130,23 @@ test_that("find_keyword() works with read_trialmaster()", {
 test_that("Extract zip without password", {
   #This would actually work as well with a random password
   target = temp_target("test_7z1")
+  unlink(target, recursive=TRUE)
   extract_7z(filename, target)
   expect_true("procformat.sas" %in% dir(target))
 })
 
 
+test_that("Extract zip subdir", {
+  target = temp_target("test_7z2")
+  unlink(target, recursive=TRUE)
+  src = test_path("CRF_Dan_Export_SAS_XPORT_2022_08_25_15_16_subdir.zip")
+  extract_7z(src, target)
+  expect_true("procformat.sas" %in% dir(target))
+  expect_true(!"PAT.xpt" %in% dir(target))
+  expect_true("PAT.xpt" %in% dir(path(target, "sub")))
+})
+
+
 ## 7z Errors ----
 
 test_that("7zip not in the path", {
@@ -186,40 +220,6 @@ test_that("expect_classed_conditions()", {
 # Misc ----------------------------------------------------------------------------------------
 
 
-test_that("fct_yesno() works", {
-  
-  set.seed(42)
-  N=20
-  x = tibble(
-    eng=sample(c("Yes", "No"), size=N, replace=TRUE),
-    fra=sample(c("Oui", "Non"), size=N, replace=TRUE),
-    bin=sample(0:1, size=N, replace=TRUE),
-    log=sample(c(TRUE, FALSE), size=N, replace=TRUE),
-    eng2=sample(c("1-Yes", "0-No"), size=N, replace=TRUE),
-
-    chr=sample(c("aaa", "bbb", "ccc"), size=N, replace=TRUE),
-    num=1:N,
-  )
-  x[10:11,] = NA
-  
-  
-  expect_snapshot({
-    
-    fct_yesno("Yes")
-    fct_yesno(c("No", "Yes"))
-    
-    mutate_all(x, fct_yesno, fail=FALSE)
-    mutate_all(x, fct_yesno, fail=FALSE, strict=TRUE)
-    mutate_all(x, fct_yesno, fail=FALSE, input=list(yes="Ja", no="Nein"))
-  })
-  
-  mutate_all(x, fct_yesno, fail=TRUE) %>% expect_error(class="fct_yesno_unparsed_error")
-  fct_yesno(x$chr) %>% expect_error(class="fct_yesno_unparsed_error")
-  # fct_yesno(x$num) %>% expect_error(class="fct_yesno_unparsed_error") #TODO?
-  fct_yesno("YesNo") %>% expect_error(class="fct_yesno_both_error")
-  fct_yesno("foobar") %>% expect_error(class="fct_yesno_unparsed_error")
-  
-})
 
 
 test_that("cli_menu() is not in package cli yet", {
@@ -229,8 +229,8 @@ test_that("cli_menu() is not in package cli yet", {
 
 
 test_that("edc_db_to_excel() works", {
-  tm = edc_example()
-  load_list(tm)
+  db = edc_example()
+  load_database(db)
   filename=tempfile(fileext=".xlsx")
   edc_db_to_excel(filename=filename, datasets=get_datasets(), open=FALSE) %>% expect_message()
   expect_true(file.exists(filename))
